####################################################################################################################################
#
#              The purpose of this script is to remove from the sample frame hexagons enough surveyable habitat to be selected by the grts function
#               But not enough to place sampling grids. This tends to be an issue for ecoregions heavily fragmented by agriculture
#               And occasionally ecoregions heavily fragnented by waterbodies.
#               Use this code for areas where the grts is likely to select too many unsurveyable hexagons to be covered by oversamples
#              This code identifies hexagons that are surveyable and unsurveyable by attempting to create 3 SSU grids in all hexagons within the area of interest
#              Very computationally expensive, could benefit from parallelism.  
#
####################################################################################################################################


library(MBHdesign)
library(spsurvey)
library(maptools)
library(rgeos)
library(raster)
library(car)
library(sf)
library(tidyverse)
library(sp)
library(rgdal)
library(lakemorpho)
library(dplyr)
library(car)
library(doBy)
library(gdata)
library(ggplot2)
library(nngeo)
library(reshape2)
library(viridis)
library(scales)




setwd("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect")




###################################################################################################################################
##########################################          IMPORT THE SAMPLING FRAME            ##########################################
###################################################################################################################################


# Albers equal-area projection parameters based on above
aea.proj <- "+proj=aea +lat_1=47.5 +lat_2=54.5 +lat_0=0 +lon_0=-113 +x_0=0 +y_0=0 +ellps=GRS80 +units=m +no_defs"
laea.proj <- "+proj=laea +lat_0=45 +lon_0=-100 +x_0=0 +y_0=0 +a=6370997 +b=6370997 +units=m +no_defs" # use this because it is lccc10 projection
tmerc.proj <- "+proj=tmerc +lat_0=0 +lon_0=-115 +k=0.9992 +x_0=500000 +y_0=0 +datum=NAD83 +units=m +no_defs" #Convert to AB hexagons projection
NAD.proj <- "+proj=utm +zone=14 +ellps=GRS80 +datum=NAD83 +units=m +no_defs " #Alberta hexagons are in this projection

hexagons <- st_read("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/hexagons.shp")
problem_regions <- c("Aspen Parkland")  ####DO NOT run this on the entire "hexagons" dataset. It is toatlly unnecessary and will take several days



hexagons_subselecion <- hexagons[hexagons$REGION_NAM %in% problem_regions,]



HabProbraster <- raster("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/habprobraster.tif")
hexagons_subselecion <-transform(hexagons_subselecion, crs(HabProbraster))

sub.rast <- crop(HabProbraster,as(hexagons,"Spatial")) # habprob raster for problem area only
 
sub1 <- sub.rast
sub1[sub1 > 0] <- 1  #set all raster pixels with a value to 1. We don't care about actual values for this process, just that they aren't 0
#sub_habvec <- rasterToPolygons(sub1)

res(sub1)    

subtotal<- raster::extract(sub1,hexagons_subselecion,fun=sum,na.rm=TRUE,df=TRUE, sp= TRUE)
subtotal <- st_as_sf(subtotal)
minpix <- 3*(600*600)/prod((res(sub1))) #minimum area required to form a 3 SSU sampling grids
maxpix <- 246   #sufficient area overwhich we don't have to worry about not forming SSUs
maybesurveyable <- subset(subtotal, habprobraster > minpix & habprobraster < maxpix ) # these are the hexagons we're concerned about
definitelysurveyable <-  subset(subtotal, habprobraster >= maxpix) #Not necessary to run the code on these hexagons so they're removed
notsurveyable <- subset(subtotal, habprobraster <= minpix) #Not necesary to run the code on these hexagons either
options(sf_max.plot=1)


sub_point_grid <- maybesurveyable %>% # create grid of points covering NWT
  st_make_grid(cellsize=300, what="centers") %>%
  st_intersection(maybesurveyable) %>%
  st_sf("geometry" = ., data.frame("point_no"=1:length(.)))

st_write(sub_point_grid, "C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect/sub_point_grid_full.shp")
sub_point_grid <- st_read("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect/sub_point_grid_full.shp")

##read in if already created###


sub_point_grid <- st_join(sub_point_grid,maybesurveyable) # add hexagon info to point nuABers


#remove some unneeded variables
sub_point_grid <- select(sub_point_grid, "point_no", "geometry","ET_Index", "REGION_NAM")
habprobs <- raster("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/habprobraster.tif")
sub_point_grid_data <- raster::extract(habprobs,sub_point_grid)
sub_point_grid$hab_prob <- sub_point_grid_data

st_write(sub_point_grid, "C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect/sub_point_grid_data.shp")
sub_point_grid <- st_read("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect/sub_point_grid_data.shp")
####################################################################################################################
#########Function to clip points to ecoregions and select grid centroids for each hex without overlap NWT###########
####################################################################################################################
###Previous function resulted in overlap between groups of 9 points


######Distance Function#####
library(data.table)
library(geosphere)

Distance <- function(point, all.points, search.dist, n){ #search function for selecting grid points around each centroid
  all.points <- all.points[all.points$ET_Index %in% point$ET_Index,] #selects only points in same hexagon of centroid
  distance <- data.frame(point_no=all.points$point_no) #creates list of points within hexagons
  for (i in 1:length(all.points$point_no)) { #measures distance between centroid and each other point in the hexagon
    distance$dist[i] <- as.numeric(st_distance(point,all.points[i,]))
    
  }
  
  distance <- subset(distance,dist < search.dist) # selects only those points within search distance of the centroid
  str(distance)
  distance <- distance[order(distance$dist),] #sorts list of points by distance
  
  return(distance)
  
}


setwd("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect")


#function to assign SSU GRIDS#
pointgrid.fn <- function(eco){      
  j=0
  eco.pointgrid <- subset(sub_point_grid,REGION_NAM==eco) #creates sf of only grid points within target ecoregion
  if(nrow(eco.pointgrid)>0){
    #eco.pointgrid<-  eco.pointgrid[ , -which(names(eco.pointgrid) %in% c("X","Y"))] #remove x &y columns
    # Group by ET_Index
    points <- subset(eco.pointgrid, hab_prob!=0) #removes pints with 0 habitat probability (probably in water)
    samplesizes <- as.data.frame(table(points$ET_Index)) #df of nuABer of remaining grid points in each hexagon
    colnames(samplesizes) <- c("ET_Index","Freq")
    nmin <- 9*3 # set min nuABer of potential point count locations per hexagon
    
    # CREATE a vector that gives the ET_Index for hexagons that have at least 18 point count locations in them
    samples<-subset(samplesizes,Freq>=nmin)
    samples<-drop.levels(samples)
    samples<-as.vector(samples$ET_Index)
    
    points<-points[points$ET_Index %in% samples,]  # removes hexagons that cannot meet minimum sample requirements
    points<-drop.levels(points) # make sure the factor levels that are empty are not included
    
    if(nrow(points)>0){
      
      ######################################################################################################################
      ### subsample based on set number of plot centroids per hexagon and stratified based on the calculated inclusion #####
      
      #### Scale the Inclusion probabilities
      sz<-4  ### NUMBER OF POTENTIAL CENTROIDS TO DRAW FOR EACH HEXAGON
      
      n<-sz*nrow(samplesizes)
      points$hab_prob <- n * (sqrt(points$hab_prob)/sum(points$hab_prob))
      
      df<-as.data.frame(points)
      df<-drop.levels(df)
      
      ### Draw the first random sample
      
      new_df <- df %>% 
        group_by(ET_Index) %>% 
        #subset(!point_no %in% redlist.V)%>% #THERE IS NO REDLIST AT THIS POINT
        sample_n(df, size=1, replace = FALSE, weight = hab_prob, .env = NULL) #selects a point from each hexagon, weighted by sample n 
      centroids <- new_df
      centroids <- ungroup(centroids)
      centroids <- st_as_sf(centroids,sf_column_name="geometry")
      centroids$centID <- centroids$ET_Index # this will later allow for point count grids going into adjacent hexes within the ecoregion
      
      
      datalist = list()
      for (i in 1:length(centroids$point_no)) {
        data <- Distance(centroids[i,],points,500,9) #applies distance function to each centroid
        datalist[[i]] <- data #creates list of centroids and surrounding points in blocks
      }
      
      #select list items that have a length >9
      #add centroids of those items to centoid redlist
      
      redlist.fn <- function(x){
        if (!is.null(x)){
          L <-nrow(x)
          if (L <=9){
            badcent <- x[which(x$dist <=1),"point_no"]
            return(badcent)
          }
        }
      }
      
      
      
      ########################
      #sort SSU draws into successful (9 points, greenlist) and unsuccessful (<9 points, redlist)
      greenlist.L = list()
      redlist.L = list()
      for (i in 1:length(centroids$point_no)) {
        data <- Distance(centroids[i,],points,500,9)
        if (nrow(data)==9){
          greenlist.L[[i]] <- data #creates list of sample blocks
        } else {
          redlist.L[[i]] <- data
          
        }
      }
      
      # If any unsuccessful grids were created, add their centroids to a list of points not to use again as centroids (redlist)
      # redlist points can still be used in other grids, just not as centroids.
      if (exists("redlist.V")) {
        redlist.V <- c(unlist(lapply(redlist.L, redlist.fn)),redlist.V)
      } else {
        redlist.V <- c(unlist(lapply(redlist.L, redlist.fn))) 
      }
      
      #only list with 9 or more items proceed:
      #combines lists into dataframe
      if (!is.null(nrow(do.call(rbind, greenlist.L)))) {
        df_total <- do.call(rbind, greenlist.L)
        point_count_grids <- points[points$point_no %in% df_total$point_no,] #selects only grid points within selected blocks
        point_count_grids$Centroid <- "N" #assigns all as Not centroid
        point_count_grids$Centroid[point_count_grids$point_no %in% centroids$point_no] <- "Y" # assigns Y only to centroids
        point_count_grids$Grid <- 1
        
        ####centroid yeilding the incomplete grid square  is dropped
        ###but the surrounding grid points are put back in the pool of availability
        
        points <- points[!points$point_no %in% point_count_grids$point_no,] # creates new point grid that excludes previously selected grids
        points <-drop.levels(points)
        
      }
      
      #create some starter variables for loop inputs
      straggler <- as.data.frame(0)
      straggler$n = 0
      stragglerA <-as.data.frame(0, nrow=2)
      complete.hexagons = 0
      
      while (straggler$n <4 & nrow(stragglerA)>0) { #repeat steps until all hexagons have 4 grids or all centroids have been tried in hexagons with <4 grids.
        j=j+1
        df<-as.data.frame(points) #points available for SSU grids
        df<-drop.levels(df)
        new_df <- df %>% #select grid centroids
          subset(!point_no %in% redlist.V)%>% #don't select from redlist
          subset(!ET_Index %in% complete.hexagons)%>% #don't select from hexagons that already have 4 grids
          group_by(ET_Index) %>% 
          sample_n(df, size=1, replace = FALSE, weight = hab_prob, .env = NULL)
        centroids <- new_df
        centroids <- ungroup(centroids)
        centroids <- st_as_sf(centroids,sf_column_name="geometry")
        centroids$centID <- centroids$ET_Index # this will later allow for point count grids going into adjacent hexes within the ecoregion
        
        #sort SSU draws into successful (9 points, greenlist) and unsuccessful (<9 points, redlist)
        greenlist.L = list()
        redlist.L = list()
        for (i in 1:length(centroids$point_no)) {
          data <- Distance(centroids[i,],points,500,9)
          if (nrow(data)==9){
            greenlist.L[[i]] <- data #creates list of sample blocks
          } else {
            redlist.L[[i]] <- data
            
          }
        }
        
        if (exists("redlist.V")) {
          redlist.V <- c(unlist(lapply(redlist.L, redlist.fn)),redlist.V)
        } else {
          redlist.V <- c(unlist(lapply(redlist.L, redlist.fn))) 
        }
        
        
        #if any 9-point grids have been successfully created, put them into a dataframe, and assign grid data
        if (!is.null(nrow(do.call(rbind, greenlist.L)))) {
          df_total <- do.call(rbind, greenlist.L)
          point_count_grids2 <- points[points$point_no %in% df_total$point_no,]
          point_count_grids2$Centroid <- "N"
          point_count_grids2$Centroid[point_count_grids2$point_no %in% centroids$point_no] <- "Y"
          point_count_grids2$Grid <- j
          
          #remove points that have been assigned to SSU grids from available point list.
          point_count_grids <- rbind(get0("point_count_grids"),get0("point_count_grids2"))
          points <- points[!points$point_no %in% point_count_grids$point_no,]
          points <-drop.levels(points)
        }
        
        #creates a list of hexagons and how many SSU grids each of them has. 
        #Skips this step if no point count grids have been created yet.
        if (exists("point_count_grids")) {
          Ngrids <- point_count_grids %>% 
            subset(Centroid == "Y") %>%
            group_by(ET_Index)%>%
            tally()
          
          
          #create inputs for "while" loop:
          straggler <-  as.data.frame(subset(Ngrids, n== min(Ngrids$n)))   #the "straggler" is the hexagon with the fewest completed SSU grids in the ecoregion.
          stragglerA <-  as.data.frame(points[ which(points$ET_Index %in% straggler$ET_Index & !points$point_no %in% redlist.V), ]) #StragglerA is a list of points in the "straggler" hexagon that haven't been tested as possible centroids.
          # loop will run until the "straggler" hexagon gets 4 complete SSU grids OR all of its points have been tested as possible SSU centroids.
          
          
          #Create list of hexagons that have 4 SSU grids, which will not be sampled in subsequent runs
          complete.hexagons <- Ngrids[Ngrids$n > 3, "ET_Index"]$ET_Index 
          
          
          
        } 
        #### some totally unnecessary outputs just so you can and see if the code looks like it's running right 
        print(eco)              
        print(j) 
        print(nrow(stragglerA))
        print(straggler$n)
        
        
      }
      
      
      write.table(st_drop_geometry(point_count_grids),"point_count_grids_sub.csv",sep=",",append=T, row.names=F,col.names=!file.exists("point_count_grids_sub.csv")) #create appended list of all point count locations
    }
  }
}

eco.list <- drop.levels(unique(maybesurveyable$REGION_NAM))
# eco.list <- c("Aspen Parkland","Boreal Transition", "Mid-Boreal Uplands") for testing only
lapply(eco.list,pointgrid.fn)
rm(i)
rm(j)

####################################################################################################################
############# Next step is matching the selected hexagons with the corresponding point count grids #################
point_count_grids_df <- read.csv("point_count_grids_sub.csv")
point_count_grids_sf <- sub_point_grid[sub_point_grid$point_no %in% point_count_grids_df$point_no,"point_no"]
point_count_grids_sf$point_no <- as.factor(point_count_grids_sf$point_no)
point_count_grids_df$point_no <- as.factor(point_count_grids_df$point_no)
point_count_grids <- left_join(point_count_grids_sf, point_count_grids_df, by = "point_no")
st_write(point_count_grids, "point_count_grids_sub.shp")

###Evaluate grid placement

Ngrids <- point_count_grids %>% 
  subset(Centroid == "Y") %>%
  group_by(ET_Index)%>%
  tally()

incomplete.Index <- Ngrids[Ngrids$n < 3, ]
complete.Index <- Ngrids[Ngrids$n > 2, ]

surveyable.Index <- as.factor(c(as.character(complete.Index$ET_Index),as.character(definitelysurveyable$ET_Index)))



surveyablehexagons <- hexagons_subselecion[hexagons_subselecion$ET_Index %in% surveyable.Index,]
unsurveyablehexagons_sub <- hexagons_subselecion[!hexagons_subselecion$ET_Index %in% surveyable.Index,]



#st_write(unsurveyablehexagons_sub, "C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selectionun/surveyablehexagons.shp")
##############
#############  Pick up PSU selection script 


#unsurveyablehexagons_sub <- read_sf("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/subselect/unsurveyablehexagons_sub.shp")

hexagons <- read_sf("C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/hexagons.shp") 


surveyable_hexagons <- hexagons[!hexagons$ET_Index %in% unsurveyablehexagons_sub$ET_Index,]
st_write(surveyable_hexagons, "C:/Users/caitlin/Desktop/WLSD/manitoba/MB_Atlas/testrun/Manitoba_selection/surveyablehexagons.shp")
